/*---------------------------------------------------------------------------------------------------*\
|   PROGRAM NAME: enrollment.sas                                                                      |
|                                                                                                     |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   Version: Revised 1.0                                                                              |
|   Date: September 10, 2015                                                                          |
|   MOD: 10/19/15 RRosofsky: Added quarterly totals                                                   |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   Program inputs:                                                                                   |
|       PatKey demographic table                                                                      |
|       Enrollment table derivative                                                                   |
|   Program outputs:                                                                                  |
|       Enrollment Summary Table                                                                      |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   Contact:                                                                                          |
|       Mini-Sentinel Coordinating Center                                                             |
|       info@mini-sentinel.org                                                                        |
\*---------------------------------------------------------------------------------------------------*/

** 1. Read from extract of Enrollment Derivative table, which meets these requirements: ;
*     A. Has only variables PatKey, Enr_Start, Enr_End, MedCov, and Drug_Cov. ;
*     B. Has only rows where: ;
*        - MedCov is either Y or N only and ;
*        - DrugCov is either Y or N only and ;
*        - Enr_Start >= Enr_End and ;
*       - all extracted variables are not missing.;
* Note that a PatKey (i.e. PatID) can be found in the Enrollment table more than once with distinct rows.;

%** MOD: 12/5/15  -RR: Within data step, created temporary variables for year of Enr_Start and Enr_End;
%** MOD: 12/8/15  -RR: Set output from TableExtract as a view;
%**               -MR  Use match-merge DS view instead of hash-join DS view to link demographics;
%**               -MR: Use temp variable _Jan1st instead of recalculating mdy(1,1,_yr) each time;
%**               -MR  Fix comment-wrapping by adding semi-colons when a new line is desired in the log;
/*
%*** Calling this macro is only to obtain the Birth_Date and Sex variables
**** Actual calculation of Age_Group occurs later in this program - Note that the ComputeAge parm is set to NO;
%TableExtract(
  MSCDMTable=DPTemp.Enr_Spans,
  TableInVars=PatKey MedCov DrugCov Enr_Start Enr_End,
  EventDate=mdy(1,1,year(Enr_Start)),
  FilterCode=%nrstr(1
    ),
  TransformCode=%nrstr(
    ),
  ComputeAge=N,
  DemHashTable=DPTemp.Patkey,
  HashVarKeys=PatKey,
  HashVarsRead=PatKey Birth_Date Sex,
  CovKeyVars=,
  OutName=EnrDemo,
  OutType=V,
  TableOutVars=PatKey Sex MedCov DrugCov Enr_Start Enr_End Birth_Date,
  Lengths=,
  SplitTime=N,
  ReplaceVarPairs=%str(PatID|PatKey)
  );
*/

** 2. Merge Enrollment with Demographics to get Birth_Date and Sex;
data EnrDemoV /view=EnrDemoV ;
  merge DPTemp.Enr_Spans (in=a keep=PatKey Enr_Start Enr_End DrugCov MedCov)
        DPTemp.PatKey (in=b keep=PatKey Birth_Date Sex);
  by PatKey;
run;

** 3. For each EnrDemoV row, calculate the value of Days as Enr_End minus Enr_Start + 1.;
** 4. For each EnrDemoV row, by PatKey, identify the different calendar years across Enr_Start through Enr_End span.;
**    Per PatKey, for each calendar year identified, create a new row and do the following:;
***** 4.1. Set the value of Year to the calendar year identified.;
***** 4.2. Calculate the number of days inclusive within the calendar year from among Days.  Set this value to a temporary variable AllDays.;
data EnrDemoByYears   (keep=PatKey Sex MedCov DrugCov Year    AllDays        AgeTimePeriodGroupKey Birth_Date)
     EnrDemoByQuarters(keep=PatKey Sex MedCov DrugCov Quarter AllDaysQuarter AgeQuarterKey Birth_Date);
  set EnrDemoV (rename=(Enr_Start=Enr_Start Enr_End=Enr_End Birth_Date=Birth_Date));
  * Create temporary variables for Enr_Start and Enr_End Years for use in loop;
  _Enr_Start_Yr=year(Enr_Start);    _Enr_End_Yr=year(Enr_End);
  do _yr=_Enr_Start_Yr to _Enr_End_Yr;
    Year=put(_yr,4.);
    * Set temporary January 1st date variable;
    _Jan1st=mdy(1,1,_yr);
    *** Count days depending on relationship of index _yr, Enr_Start, and Enr_End - It will be one of the following:;
    *** Enr_Start through Enr_End:     Enr_Start = _yr and Enr_End = _yr;
    *** Enr_Start through December 31: Enr_Start = _yr and Enr_End > _yr;
    *** January 1 through Enr_End:     Enr_Start < _yr and Enr_End = _yr;
    *** January 1 through December 31: Enr_Start < _yr and Enr_End > _yr;
    AllDays=min(mdy(12,31,_yr),Enr_End) - max(_Jan1st,Enr_Start) + 1;
    ** Calculate age and age group as of the January 1st date of the Year;
    _AgeJan1st=max(0,AgeCalc(Birth_Date,_Jan1st));
    ** Set Age Group key variable;
    AgeTimePeriodGroupkey=put(_AgeJan1st,AgeGroup_to_ID.);
    output EnrDemoByYears;
    ** Perform aggregations by quarter;
    do _qtr=1 to 4;
      Quarter= put(_Yr,4.) || "Q" || put(_qtr,1.);
      * Set 1st of quarter variable;
      _QtrBegin=intnx("QTR",_Jan1st,_qtr-1,"BEGINNING");
      _QtrEnd=intnx("QTR",_Jan1st,_qtr-1,"END");
      * Compute days per quarter;
      AllDaysQuarter=max(min(_QtrEnd,Enr_End) - max(_QtrBegin,Enr_Start) + 1,0);
      ** Calculate age and age group as of the 1st date of the quarter;
      _Age_QtrBegin=max(0,AgeCalc(Birth_Date,_QtrBegin));
      ** Set Age Group key variable;
      AgeQuarterKey=put(_Age_QtrBegin,AgeGroup_to_ID.);
      ** Output only when AllDaysQuarter > 0;
      if AllDaysQuarter > 0 then output EnrDemoByQuarters;
    end;
  end;
  format _Jan1st _QtrBegin _QtrEnd mmddyy10. ;
run;

*** 5. Aggregate the resulting rows, across the classification variables of Age_Group, Sex, Year, MedCov, DrugCov.;
***    For each combination of these variables, perform the following:;
***    5.1. Sum the number of AllDays and set DaysCovered to this value.;
***    5.2. Count the number of distinct PatIDs and set Members to this value.;
*** 6. Using the AgeTimePeriodGroupKey variable, create the variables Age_Group and Age_Group_ID.;
*** 7. Name the table Enrollment and structure table as per data dictionary.;
***    Sort by Age_Group_ID, Year, Sex, MedCov, and DrugCov.  Save to the DPLocal storage area.;
proc sql _method;
  create table DPLocal.enrollment as
  select put(AgeTimePeriodGroupkey,$AgeGroupID_to_Value.) as Age_Group length=5 format=$5. label="Age Group"
       , Sex         length=1 format=$1. label="Sex"
       , Year        length=6 format=$6. label="Year"
       , DrugCov     length=1 format=$1. label="Drug Coverage"
       , MedCov      length=1 format=$1. label="Medical Coverage"
       , DaysCovered length=8 format=15. label="Days Covered"
       , Members     length=8 format=15. label="# of Members"
       , input(put(AgeTimePeriodGroupkey,$AgeGroupID_to_True_ID.),2.) as Age_Group_ID length=3 format=2.  label="ID"
  from
  (select AgeTimePeriodGroupKey, Sex, left(Year) as Year length=6, DrugCov, MedCov, sum(AllDays) as DaysCovered, count(distinct PatKey) as Members
  from EnrDemoByYears
  group by AgeTimePeriodGroupkey, Year, Sex, MedCov, DrugCov
  union all
  select AgeQuarterKey as AgeTimePeriodGroupKey, Sex, Quarter as Year, DrugCov, MedCov, sum(AllDaysQuarter) as DaysCovered
    , count(distinct PatKey) as Members
  from EnrDemoByQuarters
  group by AgeQuarterKey, Quarter, Sex, MedCov, DrugCov)
  order by Age_Group_ID, Year, Sex, MedCov, DrugCov
  ;
quit;

* End of enrollment.sas;
