/*---------------------------------------------------------------------------------------------------*\
|   PROGRAM NAME: rx_etl.sas                                                                          |
|     Extracts, filters, and transforms the MSCDM Dispensing table                                    |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   Version: Revised 1.0                                                                              |
|   Date: September 24, 2015                                                                          |
|                                                                                                     |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   Program inputs:                                                                                   |
|       PatKey demographic table                                                                      |
|       Macro TableExtract                                                                            |
|       MSCDM Dispensing table                                                                        |
|   Program outputs:                                                                                  |
|       Multiple tables named rxYYY1, rxYYY2, rxYYY3, etc.                                            |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   Contact:                                                                                          |
|       Mini-Sentinel Coordinating Center                                                             |
|       info@mini-sentinel.org                                                                        |
\*---------------------------------------------------------------------------------------------------*/

%** MOD:10/08/15 -MR:
**                - Replace calculated year(Adate) function call with calculated _Year variable
**                - Replace CovKeyVar with CovKeyVars
**                - Use %DSN_OBS macro to get # of obs in a dataset
**                - Drop PatID from HashVarsRead parameter for performance;

%** MOD: 10/11/15 -RR: Add filtering of RxDate between DP min and max dates
**                     Add use of ReplaceVarPairs parm for enabling output SORTEDBY option;
%** MOD: 10/20/15 -RR: Add creation of variable AgeQuarterKey;
%** MOD: 10/23/15 -MR: Replace NumLength function with NumLength macro due to unresolved %sysfunc issues;
%** MOD: 12/5/15  -RR: Added parm OutType= to request static datasets;
%** MOD: 12/8/15  -RR: Made call to CovKeyVars=CovKeyDrugAndMed;
%** MOD: 12/8/15  -MR: Change to output a single view instead of time-split datasets;
%** MOD: 12/8/15  -MR: Use yy6. format to create quarter variable;

*** 1. Create an extract from the MSCDM Dispensing table, keeping the following variables in the resulting file:
***    PatID, RxDate, NDC, RxSup, and RxAmt.  Keep only rows that meet these conditions and process as indicated:
***    1.1.	The PatID and year of RxDate are found in the Prevalent_Patients table created as part of creating Enrollment Summary Table.
***    1.2. All variables are not missing.
***    1.3. RxSup is greater than or equal to 1, thus excluding rows with missing or zero values.
***    1.4. NDC is 11 digits only.
***    1.5. Create the Period variable as the calendar year and calendar quarter component of the RxDate, in the format YYYYQ#.
***    1.6. Name this file RxYYY1, RxYYY2, RxYYY3, etc.;
%TableExtract(
  MSCDMTable=indata.&DISTABLE.,
  TableInVars=PatID RxDate NDC RxSup RxAmt,
  EventDate=RxDate,
  FilterCode=(RxSup >= 1
      and CMiss(PatID, NDC)=0
      and NMiss(RxDate, RxSup, RxAMT)=0
      and length(compress(NDC,,"KD"))=11
      and &DP_MinDate. <= RxDate <= &DP_MaxDate.),
  TransformCode=%nrstr(
    Period=put(_Year,4.);
    Quarter=put(RxDate,YYQ6.);
    RxID + 1;
    AgeQuarterKey=put(max(0,AgeCalc(Birth_Date,intnx("QTR",RxDate,0,"BEGINNING"))),AgeGroup_to_ID.);
    ),
  ComputeAge=Y,
  DemHashTable=DPTemp.Patkeyscovkey,
  HashVarKeys=PatID,
  HashVarsRead=PatKey Birth_Date Sex,
  CovKeyVars=CovKeyDrugAndMed,
  OutName=DPTemp.rx,
  OutType=V,
  TableOutVars=PatKey Sex RxDate NDC RxSup RxID AgeEventGroupKey AgeTimePeriodGroupKey Period Quarter AgeQuarterKey,
  /* RxID length is dynamically assigned in setting the next parameter */
  Lengths=Quarter $6 RxID %NumLength(%DSN_OBS(indata.&DISTABLE.)),
  SplitTime=N,
  ReplaceVarPairs=%str(PatID|PatKey)
  );

 * End of rx_etl.sas;
