/*---------------------------------------------------------------------------------------------------*\
|   PROGRAM NAME: validate_inputs.sas                                                                 |
|                 Code used validate inputs                                                           |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   Version: Revised 1.0                                                                              |
|   Date: November 26, 2015                                                                           |
|                                                                                                     |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   Program inputs:                                                                                   |
|       various include files containing utility code                                                 |
|       MSCDM enrollment table                                                                        |
|   Program outputs:                                                                                  |
|       Compiled macros, formats, user functions, ready to be used                                    |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   Contact:                                                                                          |
|       Mini-Sentinel Coordinating Center                                                             |
|       info@mini-sentinel.org                                                                        |
\*---------------------------------------------------------------------------------------------------*/

%MACRO _VALIDATE_INPUTS;

%put;
%put *** Validate parameter values which are defined in qt_main_program.sas;
%put;

  %*** Set ExportPath parm to DPLOCAL if it is missing;
  %if %length(&ExportPath)=0 %then %do;
    %let ExportPath=&DPLocal.;
  %end;
  %else %do;
    %let ExportPath = %ms_clean_path(&ExportPath);
  %end;

  %*** Validate AccessFiles parm;
  %ValidateParm(parm=AccessFiles,vals=N M A, LogMessageStub=);

  %if &AccessFiles NE N %then %do;
    %*** Validate ACCESS_DBMS parm;
    %ValidateParm(parm=ACCESS_DBMS,vals=ACCESSCS ACCESS, LogMessageStub=);
  %end;

  %*** If Access export is requested, check that Common Components parm matches AccessFiles parm;
  %* Default running of Access export to No;
  %let ProcessAccess=N;

  %** If AccessFiles parm calls for generating Access database, then check Common Components to ensure that this is possible;
  %if &AccessFiles. NE N and (&ACCESS_DBMS. EQ ACCESSCS or &ACCESS_DBMS. EQ ACCESS) %then %do;
    %if &_sas_access. NE Y %then %do;
      %** Print message and set switch ProcessAccess;
      %put;
      %put ***********************************************************************************************************;
      %put *  Access Database cannot be generated as SAS/Access Interface to PC files is not indicated to be available;
      %put *  Check installation of SAS components and the setting for the _sas_Access parm in 'Common Components.sas';
      %put ***********************************************************************************************************;
      %put;
    %end;
    %else %do;
       %let ProcessAccess=Y;
    %end;
    %put &=ProcessAccess;
  %end;

  ** Save macro variables to DPTemp so that the export module may be able retrieve them;
  data DPTemp.ExportFilesMeta ;
    length name $32. value $32767.;
    name= "TextFiles"; value= "&TextFiles."; output;
    name= "ExportPath"; value= "&ExportPath."; output;
    name= "AccessFiles"; value= "&AccessFiles."; output;
    name= "ACCESS_DBMS"; value= "&ACCESS_DBMS."; output;
    name= "ProcessAccess"; value= "&ProcessAccess."; output;
  run ;

run;
%put &=ProcessAccess.;

%put;
%put *** Validate parameter values which are defined in the include "master" (e.g. 001-prod-master.sas);
%put;

  %*** Validate DEBUG parm;
  %ValidateParm(parm=DEBUG,vals=N Y, LogMessageStub=);

  %*** Validate CleanWork parm;
  %ValidateParm(parm=CleanWork,vals=N Y, LogMessageStub=);

  %*** Validate CleanDPTemp parm;
  %ValidateParm(parm=CleanWork,vals=N Y, LogMessageStub=);

  %*** Validate ModuleLogs parm;
  %ValidateParm(parm=CleanWork,vals=N Y, LogMessageStub=);

  %*** Validate TextFiles parm;
  %ValidateParm(parm=TextFiles,vals=N C P T, LogMessageStub=);

%put;
%put *** Validate that input files in package are lower case;
%put;

%get_names(type=file_name, root=%str(&sasprograms.), out=file_names_0)
%get_names(type=file_name, root=%str(&infolder.), out=file_names_1)
%get_names(type=file_name, root=%str(&infolder./module_code), out=file_names_2)
%get_names(type=file_name, root=%str(&infolder./utility_code), out=file_names_3)

data mixed_case_filenames;
  set file_names_0
      file_names_1
      file_names_2
      file_names_3;
  where anyupper(name) > 0;
run;

%if %dsn_obs(mixed_case_filenames) > 0 %then %do;
  proc sort data=mixed_case_filenames;
    by root name;
  run;

  title 'Mixed Case Filenames';
  title2 'Please change all file names listed below to lower case';
  proc print data=mixed_case_filenames;
    var type name;
    by root;
  run;

  %put;
  %put !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! ;
  %put !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! ;
  %put !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! ;
  %put ;
  %put %upcase(e)%upcase(rror:) Upper case letters detected in input file names.;
  %put Automatic macro variable SYSCC is now set to 99;
  %let syscc=99;
  %put;
  %put !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! ;
  %put !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! ;
  %put !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! ;
  %put ;

%end;

%MEND;

%_VALIDATE_INPUTS;

* End of validate_inputs.sas;
